@@ -149,21 +149,29 @@ class Application(SingletonConfigurable):
 
     # The name of the application, will usually match the name of the command
     # line application
-    name: t.Union[str, Unicode] = Unicode("application")
+    name: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode("application")
 
     # The description of the application that is printed at the beginning
     # of the help.
-    description: t.Union[str, Unicode] = Unicode("This is an application.")
+    description: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
+        "This is an application."
+    )
     # default section descriptions
-    option_description: t.Union[str, Unicode] = Unicode(option_description)
-    keyvalue_description: t.Union[str, Unicode] = Unicode(keyvalue_description)
-    subcommand_description: t.Union[str, Unicode] = Unicode(subcommand_description)
+    option_description: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
+        option_description
+    )
+    keyvalue_description: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
+        keyvalue_description
+    )
+    subcommand_description: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
+        subcommand_description
+    )
 
     python_config_loader_class = PyFileConfigLoader
     json_config_loader_class = JSONFileConfigLoader
 
     # The usage and example string that goes at the end of the help string.
-    examples: t.Union[str, Unicode] = Unicode()
+    examples: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode()
 
     # A sequence of Configurable subclasses whose config=True attributes will
     # be exposed at the command line.
@@ -190,18 +198,18 @@ class Application(SingletonConfigurable):
                     yield parent
 
     # The version string of this application.
-    version: t.Union[str, Unicode] = Unicode("0.0")
+    version: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode("0.0")
 
     # the argv used to initialize the application
     argv: t.Union[t.List[str], List] = List()
 
     # Whether failing to load config files should prevent startup
-    raise_config_file_errors: t.Union[bool, Bool] = Bool(
+    raise_config_file_errors: t.Union[bool, Bool[bool, t.Union[bool, int]]] = Bool(
         TRAITLETS_APPLICATION_RAISE_CONFIG_FILE_ERROR
     )
 
     # The log level for the application
-    log_level: t.Union[str, int, Enum] = Enum(
+    log_level: t.Union[str, int, Enum[t.Any, t.Any]] = Enum(
         (0, 10, 20, 30, 40, 50, "DEBUG", "INFO", "WARN", "ERROR", "CRITICAL"),
         default_value=logging.WARN,
         help="Set the log level by value or name.",
@@ -209,11 +217,11 @@ class Application(SingletonConfigurable):
 
     _log_formatter_cls = LevelFormatter
 
-    log_datefmt: t.Union[str, Unicode] = Unicode(
+    log_datefmt: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
         "%Y-%m-%d %H:%M:%S", help="The date format used by logging formatters for %(asctime)s"
     ).tag(config=True)
 
-    log_format: t.Union[str, Unicode] = Unicode(
+    log_format: t.Union[str, Unicode[str, t.Union[str, bytes]]] = Unicode(
         "[%(name)s]%(highlevel)s %(message)s",
         help="The Logging format template",
     ).tag(config=True)
@@ -420,11 +428,11 @@ class Application(SingletonConfigurable):
 
     _loaded_config_files = List()
 
-    show_config: t.Union[bool, Bool] = Bool(
+    show_config: t.Union[bool, Bool[bool, t.Union[bool, int]]] = Bool(
         help="Instead of starting the Application, dump configuration to stdout"
     ).tag(config=True)
 
-    show_config_json: t.Union[bool, Bool] = Bool(
+    show_config_json: t.Union[bool, Bool[bool, t.Union[bool, int]]] = Bool(
         help="Instead of starting the Application, dump configuration to stdout (as JSON)"
     ).tag(config=True)
 
@@ -436,9 +444,9 @@ class Application(SingletonConfigurable):
     def _show_config_changed(self, change):
         if change.new:
             self._save_start = self.start
-            self.start = self.start_show_config  # type:ignore[assignment]
+            self.start = self.start_show_config  # type:ignore[method-assign]
 
-    def __init__(self, **kwargs):
+    def __init__(self, **kwargs: t.Any) -> None:
         SingletonConfigurable.__init__(self, **kwargs)
         # Ensure my class is in self.classes, so my attributes appear in command line
         # options and config files.
@@ -446,7 +454,7 @@ class Application(SingletonConfigurable):
         if cls not in self.classes:
             if self.classes is cls.classes:
                 # class attr, assign instead of insert
-                self.classes = [cls] + self.classes
+                self.classes = [cls, *self.classes]
             else:
                 self.classes.insert(0, self.__class__)
 
@@ -503,12 +511,7 @@ class Application(SingletonConfigurable):
 
             for traitname in sorted(class_config):
                 value = class_config[traitname]
-                print(
-                    "  .{} = {}".format(
-                        traitname,
-                        pprint.pformat(value, **pformat_kwargs),
-                    )
-                )
+                print(f"  .{traitname} = {pprint.pformat(value, **pformat_kwargs)}")
 
     def print_alias_help(self):
         """Print the alias parts of the help."""
@@ -947,7 +950,7 @@ class Application(SingletonConfigurable):
         """Load config files by filename and path."""
         filename, ext = os.path.splitext(filename)
         new_config = Config()
-        for (config, fname) in self._load_config_files(
+        for config, fname in self._load_config_files(
             filename,
             path=path,
             log=self.log,
