@@ -2,13 +2,14 @@
 
 # Copyright (c) IPython Development Team.
 # Distributed under the terms of the Modified BSD License.
+from __future__ import annotations
 
 import logging
 
-_logger = None
+_logger: logging.Logger | None = None
 
 
-def get_logger():
+def get_logger() -> logging.Logger:
     """Grab the global logger instance.
 
     If a global Application is instantiated, grab its logger.
